@@ -1,19 +1,17 @@
-import copy
 import os
-import shutil
 import time
-import math
 import json
+import math
 import numpy as np
 import pandas as pd
 import boto3
 import torch
 import wandb
-from torch.utils.data import Dataset, IterableDataset, DataLoader
+from torch.utils.data import Dataset
 import mlflow
 import pytorch_lightning as pl
 from pytorch_lightning import LightningModule
-from rbi_ml.lightning.callbacks_base import MLflowCallbackBase, TensorboardCallbackBase
+from rbi_ml.lightning.callbacks_base import AttnVisCallbackBase, TensorboardCallbackBase
 from rbi_ml.models.txt import TxT
 
 
@@ -25,13 +23,44 @@ class TxTTensorboardCallback(TensorboardCallbackBase):
         pass
 
 
+class TxTContextAttnVisCallback(AttnVisCallbackBase):
+    def __init__(self, transformer_prefix, cols, sample_inputs, device):
+        super().__init__(transformer_prefix, cols, sample_inputs, device)
+
+    def get_attn_weights(self, pl_module):
+        ctx_embedding_layers = pl_module.txt.context_transformer.ctx_embedding
+        attn_layer = pl_module.txt.context_transformer.ctx_encoder.layers[0].self_attn
+        ctx_embedding_list = [ctx_embedding_layers[i](input_ctx.to(self.device)).unsqueeze(1)
+                              for i, input_ctx in enumerate(self.sample_inputs)]
+        ctx_out = torch.cat(ctx_embedding_list, dim=1)
+        _, attn_weight = attn_layer(ctx_out, ctx_out, ctx_out, need_weights=True)
+        avg_attn_weight = attn_weight.mean(0).mean(0).detach().tolist()
+        return avg_attn_weight
+
+
+class TxTSequenceAttnVisCallback(AttnVisCallbackBase):
+    def __init__(self, transformer_prefix, cols, sample_inputs, device):
+        super().__init__(transformer_prefix, cols, sample_inputs, device)
+
+    def get_attn_weights(self, pl_module):
+        seq_in = self.sample_inputs[0].to(self.device)
+        vl_in = self.sample_inputs[1].to(self.device)
+        sequence_transformer_layer = pl_module.txt.sequence_transformer
+        attn_layer = pl_module.txt.sequence_transformer.seq_encoder.layers[0].self_attn
+        seq_out = sequence_transformer_layer.seq_embedding(seq_in.long())
+        seq_out = seq_out * math.sqrt(sequence_transformer_layer.seq_embed_size)
+        seq_out = sequence_transformer_layer.pos_encoder(seq_out)
+        mask = sequence_transformer_layer.create_key_padding_mask(seq_in=seq_in, valid_length=vl_in)
+        _, attn_weight = attn_layer(seq_out, seq_out, seq_out, key_padding_mask=mask, need_weights=True)
+        avg_attn_weight = attn_weight.mean(0).mean(0).detach().tolist()
+        return avg_attn_weight
+
+
 class TxTWandbArtifactCallback(pl.callbacks.Callback):
     def __init__(self, wandb_run, lookup_output_dir, torch_model_params, pl_model_params, model_dir):
         self.wandb_run = wandb_run
         self.model_dir = model_dir
         self.lookup_output_dir = lookup_output_dir
-        torch_model_params["ctx_nums"] = [int(i) for i in torch_model_params["ctx_nums"]]
-        torch_model_params["seq_num"] = int(torch_model_params["seq_num"])
         self.torch_model_params = torch_model_params
         pl_model_params["loss"] = str(pl_model_params["loss"])
         pl_model_params["optimizer"] = str(pl_model_params["optimizer"])
@@ -197,16 +226,12 @@ class TxTOrderDataset(Dataset):
         return data, label
 
     def get_feature_num(self, col):
-        # TODO: implement via lookup dict
-        # data_col = self.data[col]
-        # if data_col.dtype == "int64":
-        #     return data_col.astype(int).max() + 1
         if col == self.ctx_cols[0]:
-            return self.ctx_np[:, 0].astype(int).max() + 1
+            return int(self.ctx_np[:, 0].astype(int).max() + 1)
         elif col == self.ctx_cols[1]:
-            return self.ctx_np[:, 1].astype(int).max() + 1
+            return int(self.ctx_np[:, 1].astype(int).max() + 1)
         elif col == self.ctx_cols[2]:
-            return self.ctx_np[:, 2].astype(int).max() + 1
+            return int(self.ctx_np[:, 2].astype(int).max() + 1)
         elif col == self.seq_col:
             return int(self.seq_np.reshape(-1).max()) + 1
         # elif data_col.dtype == "object":
